#include <stdio.h>
#include <stdlib.h>
#include <stm32f0xx.h>

#define APBCLK 8000000UL
#define BAUDRATE 115200UL
//#define BAUDRATE 57600UL
#define FDEL 10
#define disable_systick_int() SysTick->CTRL &= ~SysTick_CTRL_ENABLE_Msk;
// Disable SysTick interrupt
//  
static volatile uint32_t ticks = 0;
char usart_temp = 0;  //0x53

void SysTick_Handler(void) {
  ticks++;
}

void nop_delay(unsigned int i) {
  for (; i>0; i--) {
    for (int j = 0; j < FDEL; ++j) {
      __asm__ __volatile__("nop\n\t":::"memory");
    }
  }
}

/*
   можно так же до разрешения UART сбросить флаги прерываний
1. включить тактирование порта
2. установить альтернативные функции порта
3. установить режим USART
4. установить разрешение прерывания в USART
5. разрешить прерывание USART в NVIC
6. разрешить работу USART
7. установить приоритет прерывания в NVIC
*/
void usart_init(void) {
  ////
  RCC->AHBENR    |= RCC_AHBENR_GPIOAEN;         //включаем тактирование GPIOA
  RCC->APB2ENR   |= RCC_APB2ENR_USART1EN;       //включаем тактирование USART1
  // по умолчанию USART для STM32F030 A3 и A3
  GPIOA->MODER   |= GPIO_MODER_MODER9_1;        //PA9  (TX) функция для пина A9 не основная, поэтому настраиваем как альтернативную
  GPIOA->MODER   |= GPIO_MODER_MODER10_1;       //PA10 (RX) функция для пина A10 не основная, поэтому настраиваем как альтернативную
  GPIOA->OSPEEDR |= GPIO_OSPEEDER_OSPEEDR9_0;   //PA9  (TX) - 10 MHz
  // GPIOA->AFR[1]    |= 0x0110;  //
  // По первой таблице мы находим что UART1 для пинов PA9 и PA10 соответствует AF1.
  // По второму скрину видим что AF1 соответствует 0001 бинарное или 0х01.
  // Вот это самой число и надо занести в тетрады AFRH10 и AFRH9, что и делает сей код:
  GPIOA->AFR[1] = (1<<(2*4)) | (1<<(1*4));   // или что то же самое GPIOA->AFR[1]|= 0x0110; записываем 0b0001 
  // в AFRH10 и AFRH9 (UART1 для пинов PA9 и PA10 соответствует AF1, AF1 соответствует 0b0001)
  
  RCC->CFGR3  &= ~RCC_CFGR3_USART1SW;
  RCC->CFGR3  |=  RCC_CFGR3_USART1SW_0;         //System clock (SYSCLK) selected as USART1 clock
  //RCC->CFGR3  |=  RCC_CFGR3_USART1SW;         //HSI selected as USART1 clock
  
  USART1->CR1 |= USART_CR1_UE;                  //включаем USART1
  USART1->CR1 &= ~USART_CR1_M;                  //данные - 8 бит
  USART1->CR2 &= ~USART_CR2_STOP;               //1 стоп-бит
  USART1->BRR = (APBCLK+BAUDRATE/2) / BAUDRATE; //скорость usart
  USART1->CR1 |= USART_CR1_TE;                  //включаем передатчик USART1
  USART1->CR1 |= USART_CR1_RE;                  //включаем приемник USART1
  USART1->CR1 |= USART_CR1_RXNEIE;              //прерывание по приему данных
  NVIC_EnableIRQ (USART1_IRQn);                 //разрешить прерывания от USART1
  //  Вначале в USART разрешаются прерывания по определённому событию:
  //USART1->CR1  |= USART_CR1_RXNEIE;       //прерывание по приему данных
  // Затем в NVIC разрешается прерывание USART
  //NVIC_EnableIRQ (USART1_IRQn);           //разрешить прерывания от USART1
  // Затем в прерывании анализируется источник прерывания:
  //if((USART1->ISR & USART_ISR_RXNE)!=0)
  // и принимается решение что-то делать.
  //------------------------------
  //GPIOA->AFR[0] |= _VAL2FLD(GPIO_AFRL_AFSEL2, 1) | _VAL2FLD(GPIO_AFRL_AFSEL3, 1); // GPIO_AFRL_AFSEL2 - это номер ножки, число 1 это (0001: AF1)
  // Для UART A2 A3 на STM32F030

}

void USART1_IRQHandler(void) {
  if((USART1->ISR & USART_ISR_RXNE)!=0) {
    usart_temp = USART1->RDR;
  }
}

void usart_send(uint16_t Data) {
  while(!(USART1->ISR & USART_ISR_TC));
  USART1->TDR = (Data & (uint16_t)0x01FF);
}
void USART_Tx(char data) {
  USART1->TDR = data;
  while(!(USART1->ISR & USART_ISR_TC));
  USART1->ICR &= ~USART_ISR_TC;
}

static void delay(int n) {
  unsigned endTicks = ticks + n;
  while (ticks < endTicks);
}

void exti_init(void) {
  // http://dimoon.ru/obuchalka/stm32f1/programmirovanie-stm32-chast-15-vneshnie-preryivaniya-exti.html
  RCC->APB2ENR |=RCC_APB2ENR_SYSCFGCOMPEN;        // тактирование SYSCFG
  SYSCFG->EXTICR[0] = SYSCFG_EXTICR1_EXTI1_PA;    // PA0 // Bit definition for SYSCFG_EXTICR1 register @file stm32f030x6.h
  // SYSCFG->EXTICR[0] = SYSCFG_EXTICR;
  RCC->APB2ENR &= ~RCC_APB2ENR_SYSCFGCOMPEN;      // тут можно? выключить тактирование SYSCFG
  EXTI->IMR = EXTI_IMR_MR1;                       // демаскируем прервание по 1 линии
  EXTI->FTSR = EXTI_FTSR_TR1;                     // прерывание по спаду   //разрешаем срабатывание по переднему фронту
  EXTI->RTSR = EXTI_RTSR_TR1;                     // прерывание по фронту    //разрешаем срабатывание по заднему фронту 
  GPIOA->PUPDR = GPIO_PUPDR_PUPDR0_0;             // PA0 input pullup || PUPDR0 no/PUPDR0_0 pullup/PUPDR0_1 pulldown
  NVIC_EnableIRQ(EXTI0_1_IRQn);                   // разрешаем прерывание в контроллере прерываний
  //NVIC_SetPriority(EXTI0_1_IRQn,0);
}

int main(void) {
  SystemCoreClockUpdate();
  // Make sure SystemCoreClock is up-to-date
  //my_start_hse48_pll_3x16();  

  // Enable SysTick timer interrupt
  SysTick->LOAD = (SystemCoreClock / 1000) - 1;
  SysTick->VAL = 0;
  SysTick->CTRL = SysTick_CTRL_CLKSOURCE_Msk | SysTick_CTRL_TICKINT_Msk | SysTick_CTRL_ENABLE_Msk;

  //RCC->AHBENR    |= RCC_AHBENR_GPIOAEN;   // GPIOA тактирование
  usart_init();
  GPIOA->MODER   |= GPIO_MODER_MODER12_0;
  GPIOA->OSPEEDR |= GPIO_OSPEEDER_OSPEEDR12_0;  // GPIO_OSPEEDR12_HIGH ??? stm32f0.h     GPIO_OSPEEDR_OSPEEDR12_0

  
  //__enable_irq ();

  while (1) {
    /*
    uint16_t counter = 0;
    counter++;
    GPIOA->BSRR |= GPIO_BSRR_BS_12; // set
    USART_Tx('A');USART_Tx('B');USART_Tx('C');USART_Tx('D');USART_Tx('E');USART_Tx(counter);
    USART_Tx(0x0A);
    nop_delay(20000);
    GPIOA->BSRR |= GPIO_BSRR_BR_12; // reset
    USART_Tx('G');USART_Tx('H');USART_Tx('F');USART_Tx('M');USART_Tx('N');USART_Tx(counter);
    USART_Tx(0x0A);
    nop_delay(20000);
    if (counter>9999) counter=0;*/
    if (usart_temp) { 
      GPIOA->BSRR |= GPIO_BSRR_BS_12;// set
      nop_delay(20000);
      usart_temp = 0; USART_Tx('A');USART_Tx('B');USART_Tx('C');USART_Tx('D');USART_Tx('E');
      USART_Tx(0x0A);
      } 
    else  GPIOA->BSRR |= GPIO_BSRR_BR_12; // reset
  }
}


/*

*/
/*
void USART1_IRQHandler(void)
{
   if((USART1->ISR & USART_ISR_RXNE)!=0)
   {
      usart_temp = USART1->RDR;
   }
   if (USART1->ISR & USART_ISR_ORE)
   {
      USART1->ICR |= USART_ICR_ORECF;
   }
}*/

/*
// http://easyelectronics.ru/arm-uchebnyj-kurs-usart.html
// DI-HALT:
//void USART1_IRQHandler(void) {
//  if (USART1->ISR & USART_ISR_RXNE) {
//	USART1->RDR = (USART1->RDR)+1; // +1 это для прибавления эха в случае подключения TX->RX будет постоянно увеличивать на 1.
//  }
//}
В нем видно, что мы проверяем флаг RXNE, чтобы понять по какому мы, собственно, 
поводу оказались в прерывании. Делается это одной строчкой и меня искренне удивляют 
люди которые это пытаются делать то же самой функцией через SPL. ЗОЧЕМ? 
Также помним, что чтение из DR сбрасывает тот флаг, т.е. нам не нужно о нем 
заботиться. Но на счет других надо помнить.
Теперь разрешим его, подправив строчку инициализации USART:
USART1->CR1 	|= USART_CR1_UE | USART_CR1_TE | USART_CR1_RE |		// USART1 ON, TX ON, RX ON
		     USART_CR1_RXNEIE;		 			// RXNE Int ON
Разрешим прерывания в NVIC. Через функцию CMSIS. C приоритетом прерываний пока не 
заморачиваемся. Если вдруг надо, то вам в статью про NVIC: http://easyelectronics.ru/arm-uchebnyj-kurs-preryvaniya-i-nvic-prioritetnyj-kontroller-preryvanij.html
NVIC_EnableIRQ (USART1_IRQn);
И осталось разрешить глобальные прерывания. Хотя они ЕМНИП по умолчанию включены, в отличии от той же AVR.
__enable_irq ();
*/
// EOF
